@@ -278,8 +278,34 @@ _tmpl_log_state (struct tmplpro_state *state, int level)
 static
 ABSTRACT_VALUE* walk_through_nested_loops (struct tmplpro_state *state, PSTRING name) {
   int PrevHash;
-  struct ProLoopState* currentScope = getCurrentScope(&state->param->var_scope_stack);
-  ABSTRACT_VALUE* valptr= state->param->getAbstractValFuncPtr(currentScope->param_HV, name);
+  struct ProLoopState* currentScope;
+  ABSTRACT_VALUE* valptr;
+  /* Shigeki Morimoto path_like_variable_scope extension */
+  if (state->param->path_like_variable_scope) {
+    if(*(name.begin) == '/' || strncmp(name.begin, "../", 3) == 0){
+      PSTRING tmp_name;
+      int GoalHash;
+      if(*(name.begin) == '/'){
+	tmp_name.begin   = name.begin+1; // skip '/'
+	tmp_name.endnext = name.endnext;
+	GoalHash = 0;
+      }else{
+	tmp_name.begin   = name.begin;
+	tmp_name.endnext = name.endnext;
+	GoalHash = curScopeLevel(&state->param->var_scope_stack);
+	while(strncmp(tmp_name.begin, "../", 3) == 0){
+	  tmp_name.begin   = tmp_name.begin + 3; // skip '../'
+	  GoalHash --;
+	}
+      }
+      valptr = state->param->getAbstractValFuncPtr(getScope(&state->param->var_scope_stack, GoalHash)->param_HV, tmp_name);
+      return valptr;
+    }
+  }
+  /* end Shigeki Morimoto path_like_variable_scope extension */
+
+  currentScope = getCurrentScope(&state->param->var_scope_stack);
+  valptr= state->param->getAbstractValFuncPtr(currentScope->param_HV, name);
   if ((0==state->param->global_vars) || (valptr)) return valptr;
   PrevHash=curScopeLevel(&state->param->var_scope_stack)-1;
   while (PrevHash>=0) {
@@ -400,7 +426,8 @@ tag_handler_if (struct tmplpro_state *state, PSTRING name)
   iftag.tag=HTML_TEMPLATE_TAG_IF;
   iftag.vcontext=state->is_visible;
   iftag.position=state->cur_pos; /* unused */
-  if (is_var_true(state,name)) {
+  /* state->is_visible && means that we do not evaluate variable in shadow */
+  if (state->is_visible && is_var_true(state,name)) {
     iftag.value=1;
     /* state->is_visible is not touched */
   } else {
@@ -419,16 +446,16 @@ tag_handler_unless (struct tmplpro_state *state, PSTRING name)
   iftag.tag=HTML_TEMPLATE_TAG_UNLESS;
   iftag.vcontext=state->is_visible;
   iftag.position=state->cur_pos; /* unused */
-  if (is_var_true(state,name)) {
-    iftag.value=0;
-    tagstack_push(&(state->tag_stack), iftag);
-    state->is_visible=0;
-  } else {
+  /* state->is_visible && means that we do not evaluate variable in shadow */
+  if (state->is_visible && !is_var_true(state,name)) {
     iftag.value=1;
-    tagstack_push(&(state->tag_stack), iftag);
     /* state->is_visible is not touched */
+  } else {
+    iftag.value=0;
+    state->is_visible=0;
   }
-  if (debuglevel>3) tmpl_log(state,TMPL_LOG_DEBUG2,"unless:visible context =%d value=%d ",iftag.vcontext,iftag.value);
+  tagstack_push(&(state->tag_stack), iftag);
+  if (debuglevel>3) tmpl_log(state,TMPL_LOG_DEBUG2,"tag_handler_unless:visible context =%d value=%d ",iftag.vcontext,iftag.value);
 }
 
 static 
@@ -818,7 +845,7 @@ process_tmpl_tag(struct tmplpro_state *state)
 	case 'J': case 'j':		/* JS  */
 	  escape = HTML_TEMPLATE_OPT_ESCAPE_JS;
 	  break;
-	case '0': 
+	case '0': case 'N': case 'n': /* 0 or NONE */
 	  escape = HTML_TEMPLATE_OPT_ESCAPE_NO;
 	  break;
 	default:
